#!/bin/bash

# ==========
# $File: //guest/adam_morriss/scripts/findchange.sh $
# $Revision: #10 $
# $Change: 18658 $
#
# $Author: amo $
# $DateTimeTZ: 2016/03/14 05:38:56 -0700 PDT $
# ----------
# Bash script for searching change descriptions for a given string.
#
# Takes four parameters:
# status:	one of pending, shelved, submitted
# depth: 	an integer
# pattern:	the string for which you are searching
# depotpath:	a pattern for matching files, eg. //depot/path/...
#
# With this information, the script builds a 'p4 changes' command
# then searches each change for the string given. 
# ==========

stat=$1		# 'type' of changelist
deep=$2		# depth (number of changes to check)  
patt=$3		# pattern to search for ('grep -e "$patt"')
path=$4		# changes relate to this set of files

# messages
usage="$0 status depth pattern depotpath"
nocli="'p4' client not found."
nocon="'p4' not logged in or no connection available."
stats="Status: pending | shelved | submitted"
nstat="Unknown status: $stat."
nonum="Non-numeric depth: $deep"
param="Unexpected parameters provided - need four."

# no parameters given, so report 'usage'
if [ "$#" -eq 0 ]; then echo $usage; exit 0; fi

# check we have 'p4' available
p4cli=`which p4`
if [ "$?" -ne 0 ]; then echo $usage; echo $nocli; exit 1; fi

# check there's a valid connection
$p4cli login -s > /dev/null 2>&1
if [ $? -ne 0 ]; then echo $usage; echo $nocon; exit 2; fi

# expecting four parameters; error if not
if [ "$#" -ne 4 ]; then
	echo $usage
	echo $param
	exit 2
fi

# number of changes must be integer; error if not.
if ! [[ $deep =~ ^[0-9]+$ ]] ; then 
	echo $usage
	echo $nonum
	exit 3
fi

# initialise counters
total=0
found=0

# we have got this far so let's say what we're doing.
echo "Searching the last $deep changes in '$path' for descriptions containing '$patt'"

# core 'changes' command - we'll search these changes for the pattern
chgs=`$p4cli -F %change% -ztag changes -s $stat -m$deep $path`

# loop through the changes and note which ones (if any) contain the pattern
for c in $chgs; do
	testdesc=`$p4cli -F %Description% -ztag change -o $c | grep -e "$patt"`
	rc=$?; if [[ $rc == 0 ]]; then echo "Change: $c"; (( found++ )); fi 
	(( total++ ))
done
echo Searched $total, found $found
